@@ -51,6 +51,11 @@ def mean_square(mat) -> float:
         return np.sum(mat ** 2) / sz
 
 
+def norm_n2(vec) -> float:
+    vec = np.nan_to_num(vec, nan=0.0)
+    return (np.sum(vec**2))**0.5
+
+
 def relative_squared_error(mat1, mat2) -> float:
     assert mat1.shape == mat2.shape
     meansq1 = mean_square(mat1)
@@ -662,7 +667,20 @@ def generate_random_correl_matrix(size: int, corr_max: float):
     return res
 
 
-def find_orthogonal_vector(v: object) -> object:
+def generate_orthogonal_vector(v: object) -> pd.DataFrame:
+    """
+
+     generate_orthogonal_vector:
+     generate a random vector of the same size as a given non null vector v, orthogonal to v
+
+     Parameters
+     ----------
+
+     v: the vector
+
+     :returns:
+     the random vector orthogonal to v
+     """
     if type(v) is list:
         v = pd.DataFrame(v)
     n, _ = v.shape
@@ -681,15 +699,38 @@ def find_orthogonal_vector(v: object) -> object:
 
 
 def generate_vectors_bouquet(vpivot, bouquet_size: int, min_corr: float) -> pd.DataFrame:
+
     vpivot = pd.DataFrame(vpivot)
     df_bouquet = pd.DataFrame(index=vpivot.index, columns=range(bouquet_size))
+    df_bouquet.fillna(value=0.0, inplace=True)
     epsilon = (1 - min_corr)**0.5
+    n2v = norm_n2(vpivot)
     for icol in range(bouquet_size):
-        w = find_orthogonal_vector(vpivot)
-        df_bouquet.iloc[:, icol] = vpivot + epsilon * w
+        w = generate_orthogonal_vector(vpivot)
+        n2w = norm_n2(w)
+        vals = vpivot + epsilon * n2v / n2w * pd.DataFrame(w)
+        df_bouquet.loc[:, icol] = vals
     return df_bouquet
 
 
+def generate_clusters_centroids(ndim: int, max_corr: float):
+
+    assert (ndim > 1)
+    assert (0 <= max_corr < 1)
+    a = ndim / 4.0 * (1 - max_corr / 3)
+    b = 1 - max_corr
+    c = - max_corr
+    delta = b*b - 4*a*c
+    epsmax = 0.5 * (- b + delta**0.5) / a
+    df_centroids = pd.DataFrame(index=range(ndim), columns=range(ndim))
+    df_centroids.fillna(value=0.0, inplace=True)
+    for icol in range(ndim):
+        df_centroids.iloc[icol, icol] = 1
+        df_centroids.iloc[:, icol] += np.random.uniform(0.0, 2*epsmax, ndim)
+
+    return df_centroids
+
+
 def generate_clusterized_vectors(veclen: int,
                                  nclusters: int,
                                  nbvect: int,
@@ -710,6 +751,7 @@ def generate_integer_partition(ntot: int, nbpart: int, nmin: int = 1):
         res[nbpart - 1] = partsize + ntot % nbpart
     return res
 
+
 def build_correspondence_map(df_clusters):
 
     assert len(df_clusters.columns) >= 2
