@@ -237,6 +237,7 @@ class RandomFeatures ():
 
 if __name__ == "__main__":
 
-    v = np.ones(10)
+    dfc = utl.generate_clusters_centroids(ndim=10, max_corr=0.1)
+    v = dfc.iloc[:, 0]
     dfb = utl.generate_vectors_bouquet(v, 20, 0.9)
     pass
